Package: ribor
Title: An R Interface for Ribo Files
Version: 1.23.0
Authors@R: c(
    person("Michael", "Geng", email = "michaelgeng@utexas.edu", role = c("cre", "aut")),
    person("Hakan", "Ozadam", email = "hakan.ozadam@gmail.com", role = "aut"),
    person("Can", "Cenik", email = "ccenik@austin.utexas.edu", role = "aut"))
Description: The ribor package provides an R Interface for .ribo files. It provides functionality to read the .ribo file, which is of HDF5 format, and performs common analyses on its contents.
License: GPL-3
Encoding: UTF-8
Depends: R (>= 3.6.0)
biocViews: Software, Infrastructure
Imports: dplyr, ggplot2, hash, methods, rhdf5, rlang, stats, S4Vectors,
        tidyr, tools, yaml
Suggests: testthat, knitr, rmarkdown
RoxygenNote: 7.1.1
VignetteBuilder: knitr
Collate: 'annotation_functions.R' 'check_functions.R'
        'coverage_functions.R' 'ribo_class.R' 'ribo_methods.R'
        'create_ribo.R' 'helper_functions.R' 'info_functions.R'
        'metagene_functions.R' 'region_count_functions.R' 'ribor.R'
        'rnaseq_functions.R'
git_url: https://git.bioconductor.org/packages/ribor
git_branch: devel
git_last_commit: 9decab8
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
NeedsCompilation: no
Packaged: 2025-10-30 22:41:44 UTC; biocbuild
Author: Michael Geng [cre, aut],
  Hakan Ozadam [aut],
  Can Cenik [aut]
Maintainer: Michael Geng <michaelgeng@utexas.edu>
