# Generated by roxygen2: do not edit by hand

export(Ribo)
export(get_coverage)
export(get_experiments)
export(get_info)
export(get_internal_region_coordinates)
export(get_internal_region_lengths)
export(get_length_distribution)
export(get_metadata)
export(get_metagene)
export(get_original_region_coordinates)
export(get_original_region_lengths)
export(get_reference_names)
export(get_region_coordinates)
export(get_region_counts)
export(get_region_lengths)
export(get_rnaseq)
export(get_tidy_metagene)
export(plot_length_distribution)
export(plot_metagene)
export(plot_region_counts)
export(rename_default)
export(rename_transcripts)
export(set_aliases)
exportClasses(Ribo)
exportMethods(alias_hash)
exportMethods(experiment_info)
exportMethods(experiments)
exportMethods(format_version)
exportMethods(has_metadata)
exportMethods(left_span)
exportMethods(length_max)
exportMethods(length_min)
exportMethods(length_offset)
exportMethods(metagene_radius)
exportMethods(original_hash)
exportMethods(path)
exportMethods(reference)
exportMethods(right_span)
exportMethods(show)
exportMethods(transcript_info)
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,Rle)
importFrom(S4Vectors,metadata)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,desc)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,summarize)
importFrom(ggplot2,aes)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expand_limits)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,labs)
importFrom(ggplot2,position_stack)
importFrom(ggplot2,scale_fill_discrete)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(hash,has.key)
importFrom(hash,hash)
importFrom(hash,is.empty)
importFrom(hash,keys)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setClass)
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
importFrom(methods,show)
importFrom(methods,validObject)
importFrom(rhdf5,H5Fopen)
importFrom(rhdf5,h5ls)
importFrom(rhdf5,h5read)
importFrom(rhdf5,h5readAttributes)
importFrom(rlang,.data)
importFrom(stats,aggregate)
importFrom(tidyr,gather)
importFrom(tools,file_path_as_absolute)
importFrom(yaml,yaml.load)
